from itertools import islice


LF = "\n"
CRLF = "\r\n"


def count_lines(text: str) -> int:
    lf_count = text.count(LF)
    crlf_count = text.count(CRLF)
    return lf_count + crlf_count + 1


class BatchTextFileReader:
    def __init__(
        self,
        file_path: str,
        batch_size: int,
    ):
        self._file_path = file_path
        self._batch_size = batch_size

    def __enter__(self):
        self._fd = open(self._file_path)
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self._fd.close()

    def next(self) -> str:
        batch = []
        for i in range(0, self._batch_size):
            line = self._fd.readline()

            if str.isspace(line):
                continue

            if line is None:
                print("End reader")
                return ''.join(batch)

            batch.append(line)

        return ''.join(batch)


class TextSplitter:
    def __init__(
        self,
        text: str,
        batch_size: int,
    ):
        self._text = text
        self._batch_size = batch_size

    def batch(self):
        lines = self._text.splitlines(keepends=True)
        iterator = iter(lines)

        return [
            ''.join(batch)
            for batch in iter(
                lambda: list(islice(iterator, self._batch_size)),
                [],
            )
        ]



# coding: utf-8

# flake8: noqa
"""
    Vantage Management API

    This is a the API to interact with Vantage Discovery, the amazing Semantic Search Platform in the world.  We enable developers to build magical discovery experiences into their products and websites.  Some useful links: - [TODO: Semantic Search Guide: What Is It And Why Does It Matter?](https://www.bloomreach.com/en/blog/2019/semantic-search-explained-in-5-minutes)

    The version of the OpenAPI document: v1.1.2
    Contact: devrel@vantagediscovery.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from vantage_sdk.core.http.models.account import Account
from vantage_sdk.core.http.models.account_modifiable import AccountModifiable
from vantage_sdk.core.http.models.account_read_only import AccountReadOnly
from vantage_sdk.core.http.models.collection import Collection
from vantage_sdk.core.http.models.collection_immutable import (
    CollectionImmutable,
)
from vantage_sdk.core.http.models.collection_modifiable import (
    CollectionModifiable,
)
from vantage_sdk.core.http.models.collection_read_only import (
    CollectionReadOnly,
)
from vantage_sdk.core.http.models.collection_upload_url import (
    CollectionUploadURL,
)
from vantage_sdk.core.http.models.create_collection_request import (
    CreateCollectionRequest,
)
from vantage_sdk.core.http.models.document_batch import DocumentBatch
from vantage_sdk.core.http.models.embedding_search_query import (
    EmbeddingSearchQuery,
)
from vantage_sdk.core.http.models.external_key import ExternalKey
from vantage_sdk.core.http.models.external_key_modifiable import (
    ExternalKeyModifiable,
)
from vantage_sdk.core.http.models.external_key_read_only import (
    ExternalKeyReadOnly,
)
from vantage_sdk.core.http.models.facet_range import FacetRange
from vantage_sdk.core.http.models.facet_result import FacetResult
from vantage_sdk.core.http.models.ml_these import MLThese
from vantage_sdk.core.http.models.ml_these_these_inner import MLTheseTheseInner
from vantage_sdk.core.http.models.more_like_these_query import (
    MoreLikeTheseQuery,
)
from vantage_sdk.core.http.models.more_like_this_query import MoreLikeThisQuery
from vantage_sdk.core.http.models.search_options import SearchOptions
from vantage_sdk.core.http.models.search_options_collection import (
    SearchOptionsCollection,
)
from vantage_sdk.core.http.models.search_options_facets_inner import (
    SearchOptionsFacetsInner,
)
from vantage_sdk.core.http.models.search_options_field_value_weighting import (
    SearchOptionsFieldValueWeighting,
)
from vantage_sdk.core.http.models.search_options_filter import (
    SearchOptionsFilter,
)
from vantage_sdk.core.http.models.search_options_pagination import (
    SearchOptionsPagination,
)
from vantage_sdk.core.http.models.search_options_sort import SearchOptionsSort
from vantage_sdk.core.http.models.search_result import SearchResult
from vantage_sdk.core.http.models.search_result_results_inner import (
    SearchResultResultsInner,
)
from vantage_sdk.core.http.models.secondary_external_account import (
    SecondaryExternalAccount,
)
from vantage_sdk.core.http.models.semantic_search_query import (
    SemanticSearchQuery,
)
from vantage_sdk.core.http.models.shopping_assistant import ShoppingAssistant
from vantage_sdk.core.http.models.shopping_assistant_group_result import (
    ShoppingAssistantGroupResult,
)
from vantage_sdk.core.http.models.shopping_assistant_group_result_results_inner import (
    ShoppingAssistantGroupResultResultsInner,
)
from vantage_sdk.core.http.models.shopping_assistant_modifiable import (
    ShoppingAssistantModifiable,
)
from vantage_sdk.core.http.models.shopping_assistant_query import (
    ShoppingAssistantQuery,
)
from vantage_sdk.core.http.models.shopping_assistant_read_only import (
    ShoppingAssistantReadOnly,
)
from vantage_sdk.core.http.models.shopping_assistant_result import (
    ShoppingAssistantResult,
)
from vantage_sdk.core.http.models.total_count_result import TotalCountResult
from vantage_sdk.core.http.models.total_counts_options import (
    TotalCountsOptions,
)
from vantage_sdk.core.http.models.total_counts_options_total_counts import (
    TotalCountsOptionsTotalCounts,
)
from vantage_sdk.core.http.models.vantage_api_key import VantageAPIKey
from vantage_sdk.core.http.models.vantage_vibe import VantageVibe
from vantage_sdk.core.http.models.vantage_vibe_image import VantageVibeImage
from vantage_sdk.core.http.models.vantage_vibe_modifiable import (
    VantageVibeModifiable,
)
from vantage_sdk.core.http.models.vantage_vibe_read_only import (
    VantageVibeReadOnly,
)
from vantage_sdk.core.http.models.vantage_vibe_search_query import (
    VantageVibeSearchQuery,
)
from vantage_sdk.core.http.models.weighted_field_values import (
    WeightedFieldValues,
)



from typing import Any

from vantage_sdk.client import VantageClient


def create_client(
    auth_method: str,
    jwt_token: str,
    configuration: dict[str, Any],
) -> VantageClient:
    vantage_client = None

    if auth_method == "api_key":
        api_key = configuration["keys"]["vantage_api_key"]

        if api_key is None:
            raise ValueError("Vantage API key unspecified.")

        vantage_client = VantageClient.using_vantage_api_key(
            vantage_api_key=api_key,
            account_id=configuration["account"]["id"],
            api_host=configuration["api"]["api_host"],
        )
    elif auth_method == "jwt_token":
        if jwt_token is None:
            raise ValueError("JWT token unspecified.")

        vantage_client = VantageClient.using_jwt_token(
            vantage_api_jwt_token=jwt_token,
            account_id=configuration["account"]["id"],
            api_host=configuration["api"]["api_host"],
        )
    elif auth_method == "client_credentials":
        client_id = configuration["api"]["client_id"]
        client_secret = configuration["api"]["client_secret"]

        if client_id is None or client_secret is None:
            raise ValueError("Missing client credentials.")

        vantage_client = VantageClient.using_client_credentials(
            vantage_client_id=configuration["api"]["client_id"],
            vantage_client_secret=configuration["api"]["client_secret"],
            api_host=configuration["api"]["api_host"],
            auth_host=configuration["api"]["auth_host"],
            account_id=configuration["account"]["id"],
        )
    else:
        raise ValueError(
            f"Unknown auth method in $VANTAGE_AUTH_METHOD: {auth_method}"
        )

    return vantage_client



# coding: utf-8

"""
    Vantage Management API

    This is a the API to interact with Vantage Discovery, the amazing Semantic Search Platform in the world.  We enable developers to build magical discovery experiences into their products and websites.  Some useful links: - [TODO: Semantic Search Guide: What Is It And Why Does It Matter?](https://www.bloomreach.com/en/blog/2019/semantic-search-explained-in-5-minutes)

    The version of the OpenAPI document: v1.1.2
    Contact: devrel@vantagediscovery.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import io
import warnings
from typing import Any, Dict, List, Optional, Tuple, Union

from pydantic import Field, StrictFloat, StrictInt, StrictStr, validate_call


try:
    from typing import Annotated
except ImportError:
    from typing_extensions import Annotated

from typing import Optional

from pydantic import Field, StrictStr
from typing_extensions import Annotated

from vantage_sdk.core.http.api_client import ApiClient
from vantage_sdk.core.http.api_response import ApiResponse
from vantage_sdk.core.http.models.embedding_search_query import (
    EmbeddingSearchQuery,
)
from vantage_sdk.core.http.models.more_like_these_query import (
    MoreLikeTheseQuery,
)
from vantage_sdk.core.http.models.more_like_this_query import MoreLikeThisQuery
from vantage_sdk.core.http.models.search_result import SearchResult
from vantage_sdk.core.http.models.semantic_search_query import (
    SemanticSearchQuery,
)
from vantage_sdk.core.http.models.shopping_assistant_query import (
    ShoppingAssistantQuery,
)
from vantage_sdk.core.http.models.shopping_assistant_result import (
    ShoppingAssistantResult,
)
from vantage_sdk.core.http.models.total_count_result import TotalCountResult
from vantage_sdk.core.http.models.vantage_vibe_search_query import (
    VantageVibeSearchQuery,
)
from vantage_sdk.core.http.rest import RESTResponseType


class SearchApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_call
    def approximate_results_count_search(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        semantic_search_query: Annotated[
            Optional[SemanticSearchQuery],
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> TotalCountResult:
        """Approximate Results Count

        Approximate Results Count Search

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param semantic_search_query: The JSON that describes how Vantage should search a collection
        :type semantic_search_query: SemanticSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._approximate_results_count_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            semantic_search_query=semantic_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TotalCountResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def approximate_results_count_search_with_http_info(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        semantic_search_query: Annotated[
            Optional[SemanticSearchQuery],
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[TotalCountResult]:
        """Approximate Results Count

        Approximate Results Count Search

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param semantic_search_query: The JSON that describes how Vantage should search a collection
        :type semantic_search_query: SemanticSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._approximate_results_count_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            semantic_search_query=semantic_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TotalCountResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def approximate_results_count_search_without_preload_content(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        semantic_search_query: Annotated[
            Optional[SemanticSearchQuery],
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Approximate Results Count

        Approximate Results Count Search

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param semantic_search_query: The JSON that describes how Vantage should search a collection
        :type semantic_search_query: SemanticSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._approximate_results_count_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            semantic_search_query=semantic_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "TotalCountResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _approximate_results_count_search_serialize(
        self,
        account_id,
        collection_id,
        semantic_search_query,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if account_id is not None:
            _path_params['account_id'] = account_id
        if collection_id is not None:
            _path_params['collection_id'] = collection_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if semantic_search_query is not None:
            _body_params = semantic_search_query

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json']
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ['application/json']
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ['BearerAuth']

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/search/{account_id}/{collection_id}/counts',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def embedding_search(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        embedding_search_query: Annotated[
            EmbeddingSearchQuery,
            Field(
                description="The JSON that describes how Vantage should search embeddings"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SearchResult:
        """Search by embedding

        Search by embedding

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param embedding_search_query: The JSON that describes how Vantage should search embeddings (required)
        :type embedding_search_query: EmbeddingSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._embedding_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            embedding_search_query=embedding_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def embedding_search_with_http_info(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        embedding_search_query: Annotated[
            EmbeddingSearchQuery,
            Field(
                description="The JSON that describes how Vantage should search embeddings"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SearchResult]:
        """Search by embedding

        Search by embedding

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param embedding_search_query: The JSON that describes how Vantage should search embeddings (required)
        :type embedding_search_query: EmbeddingSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._embedding_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            embedding_search_query=embedding_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def embedding_search_without_preload_content(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        embedding_search_query: Annotated[
            EmbeddingSearchQuery,
            Field(
                description="The JSON that describes how Vantage should search embeddings"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Search by embedding

        Search by embedding

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param embedding_search_query: The JSON that describes how Vantage should search embeddings (required)
        :type embedding_search_query: EmbeddingSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._embedding_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            embedding_search_query=embedding_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _embedding_search_serialize(
        self,
        account_id,
        collection_id,
        embedding_search_query,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if account_id is not None:
            _path_params['account_id'] = account_id
        if collection_id is not None:
            _path_params['collection_id'] = collection_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if embedding_search_query is not None:
            _body_params = embedding_search_query

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json']
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ['application/json']
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ['BearerAuth']

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/search/{account_id}/{collection_id}/embedding',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def more_like_these_search(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        more_like_these_query: Annotated[
            MoreLikeTheseQuery,
            Field(
                description="An array, including nested these.  Each element may have only one of query_text, embedding, query_document_id or these."
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SearchResult:
        """More-Like-These(tm)

        Search for similar items, blending docs, text, and embeddings

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param more_like_these_query: An array, including nested these.  Each element may have only one of query_text, embedding, query_document_id or these. (required)
        :type more_like_these_query: MoreLikeTheseQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._more_like_these_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            more_like_these_query=more_like_these_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def more_like_these_search_with_http_info(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        more_like_these_query: Annotated[
            MoreLikeTheseQuery,
            Field(
                description="An array, including nested these.  Each element may have only one of query_text, embedding, query_document_id or these."
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SearchResult]:
        """More-Like-These(tm)

        Search for similar items, blending docs, text, and embeddings

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param more_like_these_query: An array, including nested these.  Each element may have only one of query_text, embedding, query_document_id or these. (required)
        :type more_like_these_query: MoreLikeTheseQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._more_like_these_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            more_like_these_query=more_like_these_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def more_like_these_search_without_preload_content(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        more_like_these_query: Annotated[
            MoreLikeTheseQuery,
            Field(
                description="An array, including nested these.  Each element may have only one of query_text, embedding, query_document_id or these."
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """More-Like-These(tm)

        Search for similar items, blending docs, text, and embeddings

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param more_like_these_query: An array, including nested these.  Each element may have only one of query_text, embedding, query_document_id or these. (required)
        :type more_like_these_query: MoreLikeTheseQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._more_like_these_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            more_like_these_query=more_like_these_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _more_like_these_search_serialize(
        self,
        account_id,
        collection_id,
        more_like_these_query,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if account_id is not None:
            _path_params['account_id'] = account_id
        if collection_id is not None:
            _path_params['collection_id'] = collection_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if more_like_these_query is not None:
            _body_params = more_like_these_query

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json']
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ['application/json']
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ['BearerAuth']

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/search/{account_id}/{collection_id}/morelikethese',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def more_like_this_search(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        more_like_this_query: Annotated[
            MoreLikeThisQuery,
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SearchResult:
        """More-Like-This

        Search for similar items, based on existing item id

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param more_like_this_query: The JSON that describes how Vantage should search a collection (required)
        :type more_like_this_query: MoreLikeThisQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._more_like_this_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            more_like_this_query=more_like_this_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def more_like_this_search_with_http_info(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        more_like_this_query: Annotated[
            MoreLikeThisQuery,
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SearchResult]:
        """More-Like-This

        Search for similar items, based on existing item id

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param more_like_this_query: The JSON that describes how Vantage should search a collection (required)
        :type more_like_this_query: MoreLikeThisQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._more_like_this_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            more_like_this_query=more_like_this_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def more_like_this_search_without_preload_content(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        more_like_this_query: Annotated[
            MoreLikeThisQuery,
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """More-Like-This

        Search for similar items, based on existing item id

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param more_like_this_query: The JSON that describes how Vantage should search a collection (required)
        :type more_like_this_query: MoreLikeThisQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._more_like_this_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            more_like_this_query=more_like_this_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _more_like_this_search_serialize(
        self,
        account_id,
        collection_id,
        more_like_this_query,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if account_id is not None:
            _path_params['account_id'] = account_id
        if collection_id is not None:
            _path_params['collection_id'] = collection_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if more_like_this_query is not None:
            _body_params = more_like_this_query

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json']
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ['application/json']
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ['BearerAuth']

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/search/{account_id}/{collection_id}/morelikethis',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def semantic_search(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        semantic_search_query: Annotated[
            SemanticSearchQuery,
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SearchResult:
        """Search by text

        Search by text

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param semantic_search_query: The JSON that describes how Vantage should search a collection (required)
        :type semantic_search_query: SemanticSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._semantic_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            semantic_search_query=semantic_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def semantic_search_with_http_info(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        semantic_search_query: Annotated[
            SemanticSearchQuery,
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SearchResult]:
        """Search by text

        Search by text

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param semantic_search_query: The JSON that describes how Vantage should search a collection (required)
        :type semantic_search_query: SemanticSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._semantic_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            semantic_search_query=semantic_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def semantic_search_without_preload_content(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        semantic_search_query: Annotated[
            SemanticSearchQuery,
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Search by text

        Search by text

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param semantic_search_query: The JSON that describes how Vantage should search a collection (required)
        :type semantic_search_query: SemanticSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._semantic_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            semantic_search_query=semantic_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _semantic_search_serialize(
        self,
        account_id,
        collection_id,
        semantic_search_query,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if account_id is not None:
            _path_params['account_id'] = account_id
        if collection_id is not None:
            _path_params['collection_id'] = collection_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if semantic_search_query is not None:
            _body_params = semantic_search_query

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json']
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ['application/json']
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ['BearerAuth']

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/search/{account_id}/{collection_id}/semantic',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def shopping_assistant(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        shopping_assistant_query: Annotated[
            ShoppingAssistantQuery,
            Field(
                description="JSON describing details of search using shopping assistant"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ShoppingAssistantResult:
        """Shopping Assistant

        Shopping Assistant

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param shopping_assistant_query: JSON describing details of search using shopping assistant (required)
        :type shopping_assistant_query: ShoppingAssistantQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._shopping_assistant_serialize(
            account_id=account_id,
            collection_id=collection_id,
            shopping_assistant_query=shopping_assistant_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ShoppingAssistantResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def shopping_assistant_with_http_info(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        shopping_assistant_query: Annotated[
            ShoppingAssistantQuery,
            Field(
                description="JSON describing details of search using shopping assistant"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[ShoppingAssistantResult]:
        """Shopping Assistant

        Shopping Assistant

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param shopping_assistant_query: JSON describing details of search using shopping assistant (required)
        :type shopping_assistant_query: ShoppingAssistantQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._shopping_assistant_serialize(
            account_id=account_id,
            collection_id=collection_id,
            shopping_assistant_query=shopping_assistant_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ShoppingAssistantResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def shopping_assistant_without_preload_content(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        shopping_assistant_query: Annotated[
            ShoppingAssistantQuery,
            Field(
                description="JSON describing details of search using shopping assistant"
            ),
        ],
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Shopping Assistant

        Shopping Assistant

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param shopping_assistant_query: JSON describing details of search using shopping assistant (required)
        :type shopping_assistant_query: ShoppingAssistantQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._shopping_assistant_serialize(
            account_id=account_id,
            collection_id=collection_id,
            shopping_assistant_query=shopping_assistant_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "ShoppingAssistantResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _shopping_assistant_serialize(
        self,
        account_id,
        collection_id,
        shopping_assistant_query,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if account_id is not None:
            _path_params['account_id'] = account_id
        if collection_id is not None:
            _path_params['collection_id'] = collection_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if shopping_assistant_query is not None:
            _body_params = shopping_assistant_query

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json']
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ['application/json']
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ['BearerAuth']

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/search/{account_id}/{collection_id}/shopping_assistant',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )

    @validate_call
    def vantage_vibe_search(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        vantage_vibe_search_query: Annotated[
            Optional[VantageVibeSearchQuery],
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> SearchResult:
        """Vantage Vibe

        TODO

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param vantage_vibe_search_query: The JSON that describes how Vantage should search a collection
        :type vantage_vibe_search_query: VantageVibeSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._vantage_vibe_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            vantage_vibe_search_query=vantage_vibe_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data

    @validate_call
    def vantage_vibe_search_with_http_info(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        vantage_vibe_search_query: Annotated[
            Optional[VantageVibeSearchQuery],
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[SearchResult]:
        """Vantage Vibe

        TODO

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param vantage_vibe_search_query: The JSON that describes how Vantage should search a collection
        :type vantage_vibe_search_query: VantageVibeSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._vantage_vibe_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            vantage_vibe_search_query=vantage_vibe_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )

    @validate_call
    def vantage_vibe_search_without_preload_content(
        self,
        account_id: Annotated[
            StrictStr, Field(description="The account to get information on")
        ],
        collection_id: Annotated[
            StrictStr,
            Field(
                description="The collection id to get details for {collection_id}"
            ),
        ],
        vantage_vibe_search_query: Annotated[
            Optional[VantageVibeSearchQuery],
            Field(
                description="The JSON that describes how Vantage should search a collection"
            ),
        ] = None,
        _request_timeout: Union[
            None,
            Annotated[StrictFloat, Field(gt=0)],
            Tuple[
                Annotated[StrictFloat, Field(gt=0)],
                Annotated[StrictFloat, Field(gt=0)],
            ],
        ] = None,
        _request_auth: Optional[Dict[StrictStr, Any]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Vantage Vibe

        TODO

        :param account_id: The account to get information on (required)
        :type account_id: str
        :param collection_id: The collection id to get details for {collection_id} (required)
        :type collection_id: str
        :param vantage_vibe_search_query: The JSON that describes how Vantage should search a collection
        :type vantage_vibe_search_query: VantageVibeSearchQuery
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the
                              authentication in the spec for a single request.
        :type _request_auth: dict, optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """  # noqa: E501

        _param = self._vantage_vibe_search_serialize(
            account_id=account_id,
            collection_id=collection_id,
            vantage_vibe_search_query=vantage_vibe_search_query,
            _request_auth=_request_auth,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index,
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "SearchResult",
            '405': None,
        }
        response_data = self.api_client.call_api(
            *_param, _request_timeout=_request_timeout
        )
        return response_data.response

    def _vantage_vibe_search_serialize(
        self,
        account_id,
        collection_id,
        vantage_vibe_search_query,
        _request_auth,
        _content_type,
        _headers,
        _host_index,
    ) -> Tuple:
        _host = None

        _collection_formats: Dict[str, str] = {}

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[str, str] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if account_id is not None:
            _path_params['account_id'] = account_id
        if collection_id is not None:
            _path_params['collection_id'] = collection_id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter
        if vantage_vibe_search_query is not None:
            _body_params = vantage_vibe_search_query

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json']
        )

        # set the HTTP header `Content-Type`
        if _content_type:
            _header_params['Content-Type'] = _content_type
        else:
            _default_content_type = self.api_client.select_header_content_type(
                ['application/json']
            )
            if _default_content_type is not None:
                _header_params['Content-Type'] = _default_content_type

        # authentication setting
        _auth_settings: List[str] = ['BearerAuth']

        return self.api_client.param_serialize(
            method='POST',
            resource_path='/search/{account_id}/{collection_id}/vibe',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            auth_settings=_auth_settings,
            collection_formats=_collection_formats,
            _host=_host,
            _request_auth=_request_auth,
        )



from .core.exceptions import (
    VantageException,
    VantageFileUploadError,
    VantageForbiddenError,
    VantageInvalidRequestError,
    VantageInvalidResponseError,
    VantageNotFoundError,
    VantageServiceError,
    VantageUnauthorizedError,
    VantageValueError,
)


__all__ = [
    "VantageException",
    "VantageValueError",
    "VantageNotFoundError",
    "VantageFileUploadError",
    "VantageInvalidResponseError",
    "VantageInvalidRequestError",
    "VantageUnauthorizedError",
    "VantageForbiddenError",
    "VantageServiceError",
]



"""Top-level package for Vantage SDK Python."""

from vantage_sdk.client import VantageClient
from vantage_sdk.model.account import Account
from vantage_sdk.model.collection import Collection, CollectionUploadURL
from vantage_sdk.model.keys import ExternalKey, VantageAPIKey
from vantage_sdk.model.search import (
    MoreLikeTheseItem,
    SearchResult,
    SearchResultItem,
)


__author__ = """Vantage"""
__email__ = 'none@vantage.com'
__version__ = '0.9.2'
__all__ = [
    "VantageClient",
    "Collection",
    "CollectionUploadURL",
    "Account",
    "VantageAPIKey",
    "ExternalKey",
    "SearchResult",
    "SearchResultItem",
    "MoreLikeTheseItem",
]



# coding: utf-8

"""
    Vantage Management API

    This is a the API to interact with Vantage Discovery, the amazing Semantic Search Platform in the world.  We enable developers to build magical discovery experiences into their products and websites.  Some useful links: - [TODO: Semantic Search Guide: What Is It And Why Does It Matter?](https://www.bloomreach.com/en/blog/2019/semantic-search-explained-in-5-minutes)

    The version of the OpenAPI document: v1.1.2
    Contact: devrel@vantagediscovery.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional

from pydantic import BaseModel, StrictInt

from vantage_sdk.core.http.models.ml_these_these_inner import MLTheseTheseInner
from vantage_sdk.core.http.models.search_options_collection import (
    SearchOptionsCollection,
)
from vantage_sdk.core.http.models.search_options_facets_inner import (
    SearchOptionsFacetsInner,
)
from vantage_sdk.core.http.models.search_options_field_value_weighting import (
    SearchOptionsFieldValueWeighting,
)
from vantage_sdk.core.http.models.search_options_filter import (
    SearchOptionsFilter,
)
from vantage_sdk.core.http.models.search_options_pagination import (
    SearchOptionsPagination,
)
from vantage_sdk.core.http.models.search_options_sort import SearchOptionsSort


try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class MoreLikeTheseQuery(BaseModel):
    """
    MoreLikeTheseQuery
    """  # noqa: E501

    these: Optional[List[MLTheseTheseInner]] = None
    collection: Optional[SearchOptionsCollection] = None
    request_id: Optional[StrictInt] = None
    filter: Optional[SearchOptionsFilter] = None
    field_value_weighting: Optional[SearchOptionsFieldValueWeighting] = None
    pagination: Optional[SearchOptionsPagination] = None
    sort: Optional[SearchOptionsSort] = None
    facets: Optional[List[SearchOptionsFacetsInner]] = None
    __properties: ClassVar[List[str]] = [
        "these",
        "collection",
        "request_id",
        "filter",
        "field_value_weighting",
        "pagination",
        "sort",
        "facets",
    ]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of MoreLikeTheseQuery from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in these (list)
        _items = []
        if self.these:
            for _item in self.these:
                if _item:
                    _items.append(_item.to_dict())
            _dict['these'] = _items
        # override the default output from pydantic by calling `to_dict()` of collection
        if self.collection:
            _dict['collection'] = self.collection.to_dict()
        # override the default output from pydantic by calling `to_dict()` of filter
        if self.filter:
            _dict['filter'] = self.filter.to_dict()
        # override the default output from pydantic by calling `to_dict()` of field_value_weighting
        if self.field_value_weighting:
            _dict[
                'field_value_weighting'
            ] = self.field_value_weighting.to_dict()
        # override the default output from pydantic by calling `to_dict()` of pagination
        if self.pagination:
            _dict['pagination'] = self.pagination.to_dict()
        # override the default output from pydantic by calling `to_dict()` of sort
        if self.sort:
            _dict['sort'] = self.sort.to_dict()
        # override the default output from pydantic by calling `to_dict()` of each item in facets (list)
        _items = []
        if self.facets:
            for _item in self.facets:
                if _item:
                    _items.append(_item.to_dict())
            _dict['facets'] = _items
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of MoreLikeTheseQuery from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "these": [
                    MLTheseTheseInner.from_dict(_item)
                    for _item in obj.get("these")
                ]
                if obj.get("these") is not None
                else None,
                "collection": SearchOptionsCollection.from_dict(
                    obj.get("collection")
                )
                if obj.get("collection") is not None
                else None,
                "request_id": obj.get("request_id"),
                "filter": SearchOptionsFilter.from_dict(obj.get("filter"))
                if obj.get("filter") is not None
                else None,
                "field_value_weighting": SearchOptionsFieldValueWeighting.from_dict(
                    obj.get("field_value_weighting")
                )
                if obj.get("field_value_weighting") is not None
                else None,
                "pagination": SearchOptionsPagination.from_dict(
                    obj.get("pagination")
                )
                if obj.get("pagination") is not None
                else None,
                "sort": SearchOptionsSort.from_dict(obj.get("sort"))
                if obj.get("sort") is not None
                else None,
                "facets": [
                    SearchOptionsFacetsInner.from_dict(_item)
                    for _item in obj.get("facets")
                ]
                if obj.get("facets") is not None
                else None,
            }
        )
        return _obj



#!/usr/bin/env python
from typing import Callable

import pytest

from vantage_sdk.client import VantageClient
from vantage_sdk.core.http.exceptions import ForbiddenException


""" Integration tests for account endpoints."""


class TestAccount:
    @pytest.fixture(autouse=True)
    def setup_and_teardown(
        self, client: VantageClient, account_params: dict
    ) -> None:
        """
        Takes care of setting up and tearing down test conditions.
        Runs before and after each test.
        """
        # Setup
        client.update_account(account_params["id"], account_params["id"])
        yield
        # Teardown
        client.update_account(account_params["id"], account_params["id"])

    def test_get_account(
        self, client: VantageClient, account_params: dict
    ) -> None:
        """
        Tests if retrieving user account is working correctly.
        """
        # Given
        test_account_id = account_params["id"]
        test_account_name = account_params["name"]

        # When
        account = client.get_account(test_account_id)

        # Then
        assert account.account_name == test_account_name

    def test_get_non_existing_account(
        self,
        client: VantageClient,
        account_params: dict,
        non_existing_account_id: str,
    ) -> None:
        """
        Tests if retrieving non-existing user account throws an exception.
        """

        # When
        with pytest.raises(ForbiddenException) as exception:
            client.get_account(non_existing_account_id)

        # Then
        assert exception.type is ForbiddenException

    def test_update_account(
        self,
        client: VantageClient,
        account_params: dict,
        updated_test_account_name,
    ) -> None:
        """
        Tests if updating user account is working correctly.
        """
        # Given
        test_account_id = account_params["id"]

        # When
        client.update_account(
            account_id=test_account_id, account_name=updated_test_account_name
        )

        # Then
        account = client.get_account(test_account_id)
        assert account.account_name == updated_test_account_name

    def test_update_non_existing_account(
        self,
        client: VantageClient,
        account_params: dict,
        non_existing_account_id: str,
        random_string_generator: Callable,
    ) -> None:
        """
        Tests if updating non-existing user account throws an exception.
        """
        # Given
        non_existing_account_id = non_existing_account_id
        non_existing_account_name = non_existing_account_id

        # When
        with pytest.raises(ForbiddenException) as exception:
            client.update_account(
                non_existing_account_id, non_existing_account_name
            )

        # Then
        assert exception.type is ForbiddenException



#!/usr/bin/env python
from vantage_sdk.client import VantageClient


""" Integration tests for Vantage vibe endpoints."""


class TestVantageVibe:
    def test_list_vibe_configurations(
        self, client: VantageClient, account_params: dict
    ) -> None:
        # Given
        test_account_id = account_params["id"]

        # When
        vibes_result = client.list_vibe_configurations()

        # Then
        assert len(vibes_result) == 3
        for vibe in vibes_result:
            assert vibe.account_id == test_account_id

    def test_get_vibe_configuration(
        self, client: VantageClient, account_params: dict
    ) -> None:
        # Given
        test_account_id = account_params["id"]
        test_vibe_id = "test-vibe-id"

        # When
        vibe = client.get_vibe_configuration(test_vibe_id)

        # Then
        assert vibe.account_id == test_account_id
        assert vibe.id == test_vibe_id

    def test_create_vibe_configuration(
        self, client: VantageClient, account_params: dict
    ) -> None:
        # Given
        test_account_id = account_params["id"]
        name = "test-vibe"
        external_account_id = "test-external-account-id"
        llm_model_name = "text-embeddings-ada-002"

        # When
        vibe = client.create_vibe_configuration(
            name=name,
            external_account_id=external_account_id,
            llm_model_name=llm_model_name,
        )

        # Then
        assert vibe.account_id == test_account_id
        assert vibe.id is not None
        assert vibe.name == name
        assert vibe.external_account_id == external_account_id
        assert vibe.llm_model_name == llm_model_name

    def test_delete_vibe_configuration(self, client: VantageClient) -> None:
        # Given
        test_vibe_id = "test-vibe-id"

        # When
        client.delete_vibe_configuration(test_vibe_id)

        # Then
        # Success, no error occurs
        assert 1 == 1



import json
import re
import string
from json.decoder import JSONDecodeError
from typing import Any, Optional

import pyarrow.parquet as parquet
import tiktoken

from vantage_sdk.model.validation import (
    CollectionType,
    ErrorMessage,
    ValidationError,
)


_MAX_ID_LENGTH = 256
_MAX_SEQUENCE_LENGTH = 8191
_VALID_OPERATIONS = ["add", "delete", "update"]
_VALID_META_NAME_CHARACTERS = set(
    string.ascii_lowercase + string.ascii_uppercase + string.digits + '-' + '_'
)
_VALID_META_PRIMITIVE_VALUES = (int, float, str)
_NUMBERS = (int, float)
_LIST = list


def _validate_id(document: dict[str, Any]) -> Optional[ErrorMessage]:
    if "id" not in document.keys():
        return ErrorMessage(
            field_name="id", error_message="Field must be present."
        )

    document_id = document["id"]

    if not isinstance(document_id, str):
        return ErrorMessage(
            field_name="id", error_message="Field content must be string."
        )

    if len(document_id) > _MAX_ID_LENGTH:
        return ErrorMessage(
            field_name="id",
            error_message=f"Maximum content length is {_MAX_ID_LENGTH}",
        )

    if re.search(r"[\x00-\x1F\x7F-\x9F]", document_id) is not None:
        return ErrorMessage(
            field_name="id",
            error_message="Field contains unsupported characters.",
        )

    return None


def _check_for_duplicate(
    document_id: Optional[str],
    encountered: set[str],
) -> Optional[ErrorMessage]:
    if document_id is None:
        return None

    if document_id not in encountered:
        return None

    return ErrorMessage(
        field_name="id",
        error_message="Duplicate encountered.",
    )


def _validate_text(
    document: dict[str, Any],
    model: str = None,
    mandatory: bool = True,
) -> Optional[ErrorMessage]:
    if "text" not in document.keys():
        if mandatory:
            return ErrorMessage(
                field_name="text", error_message="Field missing."
            )
        else:
            return None

    if not isinstance(document["text"], str):
        return ErrorMessage(
            field_name="text", error_message="Field content must be string."
        )

    if not model:
        return None

    encoding = tiktoken.encoding_for_model(model)

    if len(encoding.encode(document["text"])) > _MAX_SEQUENCE_LENGTH:
        return ErrorMessage(
            field_name="text",
            error_message="Content length exceeds maximum sequence length.",
        )

    return None


def _validate_operation(document: dict[str, Any]) -> Optional[ErrorMessage]:
    if "operation" not in document.keys():
        return None

    operation = document["operation"]

    if not isinstance(operation, str):
        return ErrorMessage(
            field_name="operation",
            error_message="Operation value must be string.",
        )

    if operation not in _VALID_OPERATIONS:
        return ErrorMessage(
            field_name="operation",
            error_message=f"Unsupported operation: '{operation}'.",
        )

    return None


def _is_valid_meta_name(name: str) -> bool:
    split = name.split("_", 1)

    if len(split) == 1:
        return False

    suffix = split[1]
    return set(suffix) <= _VALID_META_NAME_CHARACTERS


def _validate_meta_value(key: str, value: Any) -> ErrorMessage:
    # Value of "meta_ordered" field must be a number.
    if key.startswith("meta_ordered"):
        if not isinstance(value, _NUMBERS):
            return ErrorMessage(
                field_name=key,
                error_message="Value must be a number.",
            )
        else:
            return None

    if isinstance(value, _VALID_META_PRIMITIVE_VALUES):
        return None

    # If meta field is a list/array, check if all items are
    # either a number or a string.
    if hasattr(value, '__iter__'):
        if len(value) == 0:
            return None

        is_string_list = isinstance(value[0], str)
        for item in value:
            if not isinstance(item, _VALID_META_PRIMITIVE_VALUES):
                return ErrorMessage(
                    field_name=key,
                    error_message="Meta arrays can contain only "
                    "[str, float, int] values.",
                )
            if is_string_list:
                if not isinstance(item, str):
                    return ErrorMessage(
                        field_name=key,
                        error_message="Meta arrays cannot contain mixed values.",
                    )
            else:
                if not isinstance(item, _NUMBERS):
                    return ErrorMessage(
                        field_name=key,
                        error_message="Meta arrays cannot contain mixed values.",
                    )
        return None

    return ErrorMessage(
        field_name=key,
        error_message="Value must be either [int, str, float], or an "
        "array of one of those types.",
    )


def _validate_meta_fields(document: dict[str, Any]) -> list[ErrorMessage]:
    errors = []
    meta_fields = {
        name: value
        for name, value in document.items()
        if name == "meta" or name.startswith("meta_")
    }

    if not any(meta_fields):
        return []

    for meta_field in meta_fields.items():
        name, value = meta_field
        if not _is_valid_meta_name(name):
            errors.append(
                ErrorMessage(
                    field_name=name,
                    error_message="Field has invalid name. "
                    "May contain only [a-zA-Z0-9-_] characters.",
                )
            )
            continue

        meta_error = _validate_meta_value(name, value)
        if meta_error is not None:
            errors.append(meta_error)

    return errors


def _validate_embeddings(
    document: dict[str, Any],
    embeddings_dimension: Optional[int],
    mandatory=False,
) -> Optional[ErrorMessage]:
    if "embeddings" not in document.keys():
        if mandatory:
            return ErrorMessage(
                field_name="embeddings",
                error_message="Field must be present.",
            )
        else:
            return None

    embeddings = document["embeddings"]

    # Embeddings must be iterable (array/list)
    if not hasattr(embeddings, '__iter__'):
        return ErrorMessage(
            field_name="embeddings",
            error_message="Embeddings must be a list of float numbers.",
        )

    if not embeddings_dimension:
        return None

    actual_size = len(embeddings)
    if actual_size != embeddings_dimension:
        return ErrorMessage(
            field_name="embeddings",
            error_message="Mismatch in embeddings dimension: "
            f"found {actual_size},"
            f" expected {embeddings_dimension}",
        )

    for item in embeddings:
        if not isinstance(item, _NUMBERS):
            return ErrorMessage(
                field_name="embeddings",
                error_message="Embeddings must not contain non-number values.",
            )

    return None


def _create_json_parsing_error(
    exception: JSONDecodeError,
    line_number: int,
) -> ValidationError:
    message = f"JSON decoder error: {exception.msg}"

    return ValidationError(
        document_id=None,
        line_number=line_number,
        errors=[
            ErrorMessage(
                field_name=None,
                error_message=message,
            )
        ],
    )


class DocumentValidator:
    """Component for validating documents."""

    _encountered_ids: set[str] = set()

    def _validate_document(
        self,
        document: dict[str, Any],
        line_number: int,
        collection_type: CollectionType,
        embeddings_dimension: Optional[int] = None,
        model: Optional[str] = None,
    ) -> Optional[ValidationError]:
        document_id = document.get("id")
        error_messages = []

        id_error = _validate_id(document)
        if id_error is not None:
            error_messages.append(id_error)

        duplicate_document_error = _check_for_duplicate(
            document_id=document_id,
            encountered=self._encountered_ids,
        )
        if duplicate_document_error is not None:
            error_messages.append(duplicate_document_error)

        operation_error = _validate_operation(document)
        if operation_error is not None:
            error_messages.append(operation_error)

        text_error = _validate_text(
            document=document,
            model=model,
            mandatory=(
                collection_type == CollectionType.USER_PROVIDED_EMBEDDINGS
            ),
        )
        if text_error is not None:
            error_messages.append(text_error)

        meta_errors = _validate_meta_fields(document)
        if any(meta_errors):
            error_messages.extend(meta_errors)

        embeddings_error = _validate_embeddings(
            document=document,
            embeddings_dimension=embeddings_dimension,
            mandatory=(
                collection_type == CollectionType.USER_PROVIDED_EMBEDDINGS
            ),
        )
        if embeddings_error is not None:
            error_messages.append(embeddings_error)

        if document_id is not None:
            self._encountered_ids.add(document_id)

        if not any(error_messages):
            return None

        return ValidationError(
            document_id=document_id,
            line_number=line_number,
            errors=error_messages,
        )

    def validate_jsonl(
        self,
        file_path: str,
        collection_type: CollectionType,
        model: Optional[str] = None,
        embeddings_dimension: Optional[int] = None,
    ) -> list[ValidationError]:
        """Validates documents from a JSONL file.

        Parameters
        ----------
        file_path : str
            Path of the JSONL file in the filesystem.
        collection_type : CollectionType
            For what kind of collection are documents from this file intended.
        model : Optional[str] = None
            Which model should be used to generate embeddings (if any).
        embeddings_dimension : Optional[int] = None
            Dimension of embeddings (if provided in file).

        Raises
        ------
        FileNotFoundError
            If specified file is not found.

        Returns
        -------
        List of encountered errors. If file is valid, the list will be empty.
        """
        errors = []
        line_number = 0
        with open(file_path, 'r') as file:
            document = None
            line = None
            while True:
                line = file.readline()
                if not line:
                    # Reached end of file.
                    break

                document = None
                try:
                    document = json.loads(line)
                except JSONDecodeError as exception:
                    errors.append(
                        _create_json_parsing_error(
                            exception,
                            line_number,
                        )
                    )
                    continue

                error = self._validate_document(
                    document=document,
                    line_number=line_number,
                    model=model,
                    collection_type=collection_type,
                    embeddings_dimension=embeddings_dimension,
                )
                if error is not None:
                    errors.append(error)
                line_number += 1

        return errors

    """
    Validates documents from a Parquet file.

    Parameters
    ----------
    file_path : str
        Path of the Parquet file in the filesystem.
    collection_type : CollectionType
        For what kind of collection are documents from this file intended.
    model : Optional[str] = None
        Which model should be used to generate embeddings (if any).
    embeddings_dimension : Optional[int] = None
        Dimension of embeddings (if provided in file).

    Raises
    ------
    FileNotFoundError
        If specified file is not found.

    Returns
    -------
    List of encountered errors. If file is valid, the list will be empty.
    """

    def validate_parquet(
        self,
        file_path: str,
        collection_type: CollectionType,
        model: Optional[str] = None,
        embeddings_dimension: Optional[int] = None,
    ) -> list[ValidationError]:
        parquet_file = parquet.ParquetFile(file_path)
        batches = parquet_file.iter_batches(batch_size=4096)
        line_number = 0
        errors = []

        for batch in batches:
            items = batch.to_pandas().to_dict(orient="records")
            for document in items:
                error = self._validate_document(
                    document=document,
                    line_number=line_number,
                    model=model,
                    collection_type=collection_type,
                    embeddings_dimension=embeddings_dimension,
                )
                if error is not None:
                    errors.append(error)
                line_number += 1

        return errors


VALIDATOR = DocumentValidator()



# coding: utf-8

"""
    Vantage Management API

    This is a the API to interact with Vantage Discovery, the amazing Semantic Search Platform in the world.  We enable developers to build magical discovery experiences into their products and websites.  Some useful links: - [TODO: Semantic Search Guide: What Is It And Why Does It Matter?](https://www.bloomreach.com/en/blog/2019/semantic-search-explained-in-5-minutes)

    The version of the OpenAPI document: v1.1.2
    Contact: devrel@vantagediscovery.com
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations

import json
import pprint
import re  # noqa: F401
from typing import Any, ClassVar, Dict, List, Optional

from pydantic import BaseModel, StrictStr


try:
    from typing import Self
except ImportError:
    from typing_extensions import Self


class ShoppingAssistantModifiable(BaseModel):
    """
    ShoppingAssistantModifiable
    """  # noqa: E501

    name: Optional[StrictStr] = None
    groups: Optional[List[StrictStr]] = None
    external_account_id: Optional[StrictStr] = None
    llm_model_name: Optional[StrictStr] = None
    system_prompt_id: Optional[StrictStr] = None
    __properties: ClassVar[List[str]] = [
        "name",
        "groups",
        "external_account_id",
        "llm_model_name",
        "system_prompt_id",
    ]

    model_config = {
        "populate_by_name": True,
        "validate_assignment": True,
        "protected_namespaces": (),
    }

    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ShoppingAssistantModifiable from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        _dict = self.model_dump(
            by_alias=True,
            exclude={},
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Dict) -> Self:
        """Create an instance of ShoppingAssistantModifiable from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate(
            {
                "name": obj.get("name"),
                "groups": obj.get("groups"),
                "external_account_id": obj.get("external_account_id"),
                "llm_model_name": obj.get("llm_model_name"),
                "system_prompt_id": obj.get("system_prompt_id"),
            }
        )
        return _obj



#!/usr/bin/env python
from vantage_sdk.client import VantageClient


""" Integration tests for shopping assistant endpoints."""


class TestShoppingAssistant:
    def test_list_shopping_assistants(
        self, client: VantageClient, account_params: dict
    ) -> None:
        # Given
        test_account_id = account_params["id"]

        # When
        shopping_assistants_result = client.list_shopping_assistants()

        # Then
        assert len(shopping_assistants_result) == 3
        for assistant in shopping_assistants_result:
            assert assistant.account_id == test_account_id
            assert assistant.groups is not None

    def test_get_shopping_assistant(
        self, client: VantageClient, account_params: dict
    ) -> None:
        # Given
        test_account_id = account_params["id"]
        test_shopping_assistant_id = "test-shopping-assistant-id"

        # When
        shopping_assistant = client.get_shopping_assistant(
            test_shopping_assistant_id
        )

        # Then
        assert shopping_assistant.account_id == test_account_id
        assert (
            shopping_assistant.shopping_assistant_id
            == test_shopping_assistant_id
        )

    def test_create_shopping_assistant(
        self, client: VantageClient, account_params: dict
    ) -> None:
        # Given
        test_account_id = account_params["id"]
        name = "test-assistant"
        groups = ["group 1", "group 2", "group 3"]
        external_account_id = "test-external-account-id"
        llm_model_name = "text-embeddings-ada-002"

        # When
        shopping_assistant = client.create_shopping_assistant(
            name=name,
            groups=groups,
            external_account_id=external_account_id,
            llm_model_name=llm_model_name,
        )

        # Then
        assert shopping_assistant.account_id == test_account_id
        assert shopping_assistant.shopping_assistant_id is not None
        assert shopping_assistant.name == name
        assert shopping_assistant.groups == groups
        assert shopping_assistant.external_account_id == external_account_id
        assert shopping_assistant.llm_model_name == llm_model_name

    def test_delete_shopping_assistant(self, client: VantageClient) -> None:
        # Given
        test_shopping_assistant_id = "test-shopping-assistant-id"

        # When
        client.delete_shopping_assistant(test_shopping_assistant_id)

        # Then
        # Success, no error occurs
        assert 1 == 1
